<?php

/*
Plugin Name: Free Unsigned Music
Plugin URI: http://www.unsignedbandweb.com
Description: Add a free unsigned music widget to your blog from <a href="http://www.unsignedbandweb.com/">Unsigned Band Web</a> - Just activate then move the widget to your blog
Author: Skulled
Version: 1.0
*/


/*
*	Widget Init
*/

function widget_wpubwWidgetInit ()
{
    // Register the sidebar widget //
    register_sidebar_widget( __( "Free Music" ), "widget_wpubwWidget", "ubw-feed-widget" );

    // Register Widget Control //
    register_widget_control( "Free Music", "widget_wpubwWidget_control" );
}

// Add action to init the plugin and its control //
add_action( "plugins_loaded", "widget_wpubwWidgetInit" );


/*
*	Widget Core Function
*/

function widget_wpubwWidget ( $functionArguments )
{
    // Extract the arguments passed to the function //
    extract( $functionArguments );

    // Create display String //
    $displayString = '';

    // Lets get the widget options //
    $ubwOptions = get_option( "widget_ubw" );

    // Make sure we got something valid //
    if ( $ubwOptions != FALSE && is_object( $ubwOptions ) && isset( $ubwOptions->title ) )
    {
        $displayString .= $before_widget;
        $displayString .= $before_title . $ubwOptions->title . $after_title;

        // Create the output now //
        if ( $ubwOptions->feed_type == "genre" )
            $displayString .= widget_wpubwGetGenreData( $ubwOptions );
        else if ( $ubwOptions->feed_type == "artist" )
            $displayString .= widget_wpubwGetArtistData( $ubwOptions );

        $displayString .= $after_widget;
    }

    // Display the output //
    echo $displayString;
}


/*
*	Widget Control Function
*/

function widget_wpubwWidget_control ()
{
    // Include the rss file for rss functions //
    require_once ABSPATH . "wp-includes/rss.php";

    // Let handle all the XML stuff from the server //
    $xmlParser = xml_parser_create( "ISO-8859-1" );
    xml_parser_set_option( $xmlParser, XML_OPTION_CASE_FOLDING, 0 );
    xml_parser_set_option( $xmlParser, XML_OPTION_SKIP_WHITE, 1 );
    $xmlData = file_get_contents( "http://www.unsignedbandweb.com/music/master_xml.php" );
    xml_parse_into_struct( $xmlParser, $xmlData, $dataStruct, $dataIndex );
    xml_parser_free( $xmlParser );

    // Setup master data array //
    $feedDataArray = array ();

    // Loop through all the occurences of station tag index values //
    foreach ( $dataIndex["station"] as $index=>$stationIndex )
    {
        // Check if this occurence is a start tag or a close tag //
        if ( $dataStruct[$stationIndex]['type'] == "open" )
        {
            // Setup array for this genre type //
            $feedDataArray[$dataStruct[$stationIndex+1]['value']] = array ();
            // Charts Feed URL for Genre //
            if ( in_array( ($stationIndex+4), $dataIndex["station1feed"] ) )
                $feedDataArray[$dataStruct[$stationIndex+1]['value']]["charts"] = $dataStruct[$stationIndex+4]['value'];
            // Editors Pick Feed URL for Genre //
            if ( in_array( ($stationIndex+7), $dataIndex["station2feed"] ) )
                $feedDataArray[$dataStruct[$stationIndex+1]['value']]["editor"] = $dataStruct[$stationIndex+7]['value'];
            // New Feed URL for Genre //
            if ( in_array( ($stationIndex+10), $dataIndex["station3feed"] ) )
                $feedDataArray[$dataStruct[$stationIndex+1]['value']]["new"] = $dataStruct[$stationIndex+10]['value'];
        }
    }

    // Handle the post request if any //
    if ( isset($_POST) && !empty( $_POST ) )
    {
        // Check for the widget specific fields in post //
        if ( isset( $_POST['ubwWidgetSubmit-submit'] ) && $_POST['ubwWidgetSubmit-submit'] == '1' )
        {
            // Create the options array to update in the database //
            $title = strip_tags( trim( $_POST['ubwWidgetTitle'] ) );
            $feedType = trim( $_POST['ubwWidgetFeedType'] );
            $genreType = trim( $_POST['ubwWidgetGenreType'] );
            $genreFeedType = trim( $_POST['ubwWidgetGenreFeedType'] );
            $artistId = intval( strip_tags( trim( $_POST['ubwWidgetArtistId'] ) ) );
            $displayImage = intval( trim( $_POST['ubwWidgetDisplayImage'] ) );
            $displayLimit = intval( trim( $_POST['ubwWidgetDisplayLimit'] ) );

            // Create Options Object //
            $optionsObject = new stdClass;
            $optionsObject->title = $title;
            $optionsObject->feed_type = $feedType;
            $optionsObject->genre_type = $genreType;
            $optionsObject->genre_feed_type = $genreFeedType;
            $optionsObject->artist_id = $artistId;
            $optionsObject->display_image = $displayImage;
            if ( $displayLimit > 20 )
                $optionsObject->display_limit = 20;
            else
                $optionsObject->display_limit = $displayLimit;

            // Set feed URL //
            if ( $feedType == "genre" )
                $optionsObject->feed_url = $feedDataArray[$genreType][$genreFeedType];
            else if ( $feedType == "artist" )
                $optionsObject->feed_url = "http://www.unsignedbandweb.com/music/bands/" . $optionsObject->artist_id . "/rss.xml";

            // Update the option in the database //
            update_option( "widget_ubw", $optionsObject );

            // Delete the cache option if one exists //
            delete_option( "widget_ubw_datacache" );
        }
    }

    // Lets get the widget options //
    $ubwOptions = get_option( "widget_ubw" );

    // Make sure we got something valid //
    if ( $ubwOptions == FALSE || !is_object( $ubwOptions ) )
    {
        $ubwOptions = new stdClass;
        $ubwOptions->title = 'Free Music';
        $ubwOptions->feed_type = "genre";
        $ubwOptions->genre_type = "Acoustic";
        $ubwOptions->genre_feed_type = "charts";
        $ubwOptions->artist_id = '';
        $ubwOptions->display_image = 1;
        $ubwOptions->display_limit = 5;
    }

    // Create the Genre List //
    $genreListOptions = '';

    // Loop through all genres //
    foreach ( $feedDataArray as $genreName=>$genreUrls )
    {
        // Add Genre to List //
        $genreListOptions .= '<option value="' . $genreName . '"' . ( $ubwOptions->genre_type == $genreName ? ' selected="selected"' : '' ) . '>' . $genreName . '</option>';
    }

    // Create form for the widget control //
    $formString = '';

    // Title Field //
    $formString .= '<p style="text-align:left;">
                        <label for="ubwWidgetTitle">
                            Title:
                            <input class="widefat" id="ubwWidgetTitle" name="ubwWidgetTitle" value="' . $ubwOptions->title . '" type="text" />
                        </label>
                </p><p style="text-align:left; background: #FFF;">
                        <label for="ubwWidgetFeedType">
                            Feed Type:
                            <select name="ubwWidgetFeedType" id="ubwWidgetFeedType" class="widefat">
                                <option value="genre"' . ( $ubwOptions->feed_type == "genre" ? ' selected="selected"' : '' ) . '>Genre</option>
                                <option value="artist"' . ( $ubwOptions->feed_type == "artist" ? ' selected="selected"' : '' ) . '>Artist</option>
                            </select>
                        </label>
               </p><p style="text-align:left; background: #FFF;">
                        <label for="ubwWidgetGenreType">
                            Genre: (Use if feed type is set to Genre)</small>
                            <select name="ubwWidgetGenreType" id="ubwWidgetGenreType" class="widefat">
                            ' . $genreListOptions . '
                            </select>
                           
                        </label>
              </p><p style="text-align:left; background: #FFF;">
                        <label for="ubwWidgetGenreFeedType">
                            Genre Feed Type: (If feed type is set to genre)
                            <select name="ubwWidgetGenreFeedType" id="ubwWidgetGenreFeedType" class="widefat">
                                <option value="editor"' . ( $ubwOptions->genre_feed_type == "editor" ? ' selected="selected"' : '' ) . '>Editors Pick</option>
                                <option value="charts"' . ( $ubwOptions->genre_feed_type == "charts" ? ' selected="selected"' : '' ) . '>Charts</option>
                                <option value="new"' . ( $ubwOptions->genre_feed_type == "new" ? ' selected="selected"' : '' ) . '>New Songs</option>
                            </select>
                        </label>
                </p><p style="text-align:left; background: #FFF;">
                        <label for="ubwWidgetArtistId">
                            Artist ID: (If feed type is set to Artist)
                            <input class="widefat" id="ubwWidgetArtistId" name="ubwWidgetArtistId" value="' . $ubwOptions->artist_id . '" type="text" />
                        </label>
                 </p><p style="text-align:left; background: #FFF;">
                        <label for="ubwWidgetDisplayImage">
                            Display Song Image?:
                            <select name="ubwWidgetDisplayImage" id="ubwWidgetDisplayImage" class="widefat">
                                <option value="1"' . ( $ubwOptions->display_image == 1 ? ' selected="selected"' : '' ) . '>Yes</option>
                                <option value="0"' . ( $ubwOptions->display_image == 0 ? ' selected="selected"' : '' ) . '>No</option>
                            </select>
                        </label>
                </p><p style="text-align:left; background: #FFF;">
                        <label for="ubwWidgetDisplayLimit">
                            Number of items to display: (20 Max)
                            <input class="widefat" id="ubwWidgetDisplayLimit" name="ubwWidgetDisplayLimit" value="' . $ubwOptions->display_limit . '" type="text" />
                        </label>
                    </p>
                    <input id="ubwWidgetSubmit-submit" name="ubwWidgetSubmit-submit" value="1" type="hidden">';

    // Show Cotrol Form //
    echo $formString;
    echo "";
}


/*
*	Widget Get Genre Data from Server Feed
*/

function widget_wpubwGetGenreData ( $options )
{
    // Get the cache stuff from the DB //
    $cacheOption = get_option( "widget_ubw_datacache" );

    if ( $cacheOption != FALSE && is_object( $cacheOption ) && isset( $cacheOption->time ) )
    {
        // Set the data array from cache //
        $genreDataArray = $cacheOption->data;
        $descriptionContent = $cacheOption->description;
        $stationUrl = $cacheOption->station_url;
    }
    // Setup dummy & don't go on cache //
    else
    {
        $cacheOption = new stdClass;
        $cacheOption->time = time() - 7200;
    }

    // Setup one hour back //
    $oneHourBack = time() - 3600;

    if ( $cacheOption->time <= $oneHourBack )
    {
        // Let handle all the XML stuff from the server //
        $xmlParser = xml_parser_create( "ISO-8859-1" );
        xml_parser_set_option( $xmlParser, XML_OPTION_CASE_FOLDING, 0 );
        xml_parser_set_option( $xmlParser, XML_OPTION_SKIP_WHITE, 1 );
        $xmlData = file_get_contents( $options->feed_url );
        xml_parse_into_struct( $xmlParser, $xmlData, $dataStruct, $dataIndex );
        xml_parser_free( $xmlParser );

        // Set Description //
        $descriptionContent = $dataStruct[$dataIndex["description"][0]]['value'];

        // Set Station Url //
        $stationUrl = $dataStruct[$dataIndex["link"][0]]['value'];

        // Set up the data nicely in an array first //
        $genreDataArray = array ();

        // Loop through all the occurences of item tag index values //
        foreach ( $dataIndex["item"] as $itemOpenIndex=>$itemIndex )
        {
            // Check if this occurence is a start tag or a close tag //
            if ( $dataStruct[$itemIndex]['type'] == "open" )
            {
                // Get the index where this item closes, this is generally the next occurence of the tag //
                $itemCloseIndex = $dataIndex["item"][$itemOpenIndex+1];

                // Setup temp item array //
                $tempItemArray = array ();

                // Loop through all the index between the item open and close and add that to the data array //
                for ( $i=$itemOpenIndex+1; $i<$itemCloseIndex; $i++ )
                {
                    // Add the data to the item array //
                    $tempItemArray[$dataStruct[$i]['tag']] = $dataStruct[$i]['value'];
                }

                // Add the item to the data array //
                $genreDataArray[] = $tempItemArray;
            }
        }

        // Now set the new cached data //
        delete_option( "widget_ubw_datacache" );

        // Create new object to add to database //
        $newCache = new stdClass;
        $newCache->time = time();
        $newCache->data = $genreDataArray;
        $newCache->station_url = $stationUrl;
        $newCache->description = $dataStruct[$dataIndex["description"][0]]['value'];

        // Update the option in the database //
        update_option( "widget_ubw_datacache", $newCache );
    }

    // Create the return string with all the data //
    $returnString = '';
    

    // Start the Listing //
    $returnString .= '<ul class="ubwWidgetList">';

    $returnString .= '<li class="ubwWidgetDescription">feed: <a href="' . $stationUrl . '">' . $descriptionContent . '</a></li>';

    // Loop through all the items we collected and get the required amount //
    foreach ( $genreDataArray as $index=>$item )
    {
        // Add only as many items as needed and setup in the options //
        if ( ( $index+1 ) <= $options->display_limit )
        {
            // Add the item to the list //
            $returnString .= '<li>
                                ' . ( $options->display_image == 1 ? '<img src="http://www.unsignedbandweb.com/music/image.php?mode=song_image&band_id=' . $item['bandid'] . '&song_id=' . $item['songid'] . '&width=40&height=40" class="ubwWidgetImage" alt="' . $item['bandname'] . '" />' : '' ) . '
                                <a href="' . $item['songplaylink'] . '" title="Play Song - ' . $item['songname'] . '">"' .  substr($item['songname'], 0, 15) . '"</a>
                                <br />
                                <a href="' . $item['bandurl'] . '" title="Go to the website for the band - ' . $item['bandname'] . '"><i>' . substr($item['bandname'], 0, 15) . '</i></a>
                                <br />
                                <a class="playlink" href="' . $item['songplaylink'] . '" title="Play Song - ' . $item['songname'] . '">[ Play ]</a>&nbsp;<a class="playlink" onclick="windowOpen(this.href, \'name\', \'430\', \'500\', \'yes\'); return false;" href="' . $item['songcommentpopup400x400'] . '" title="Comments for Song - ' . $item['songname'] . '">[ Talk ]</a>     
                            </li>';
        }
        // Reached the limit for display, break out //
        else
            break;
    }

    // Add link back //
    $returnString .= '<li class="ubwWidgetLinkBack">Powered By <a href="http://www.unsignedbandweb.com/">Free Music</a></li>';

    // End the list //
    $returnString .= '</ul>';

    // Return the data now to be displayed in the widget //
    return $returnString;
}


/*
*	Widget Get Artist Data from Server Feed
*/

function widget_wpubwGetArtistData ( $options )
{
    // Get the cache stuff from the DB //
    $cacheOption = get_option( "widget_ubw_datacache" );

    if ( $cacheOption != FALSE && is_object( $cacheOption ) && isset( $cacheOption->time ) )
    {
        // Set the data array from cache //
        $artistDataArray = $cacheOption->data;
        $artistName = $cacheOption->artist_name;
        $artistUrl = $cacheOption->artist_url;
        $descriptionContent = $cacheOption->description;
    }
    // Setup dummy & don't go on cache //
    else
    {
        $cacheOption = new stdClass;
        $cacheOption->time = time() - 7200;
    }

    // Setup one hour back //
    $oneHourBack = time() - 3600;

    if ( $cacheOption->time <= $oneHourBack )
    {
        // Let handle all the XML stuff from the server //
        $xmlParser = xml_parser_create( "ISO-8859-1" );
        xml_parser_set_option( $xmlParser, XML_OPTION_CASE_FOLDING, 0 );
        xml_parser_set_option( $xmlParser, XML_OPTION_SKIP_WHITE, 1 );
        $xmlData = file_get_contents( $options->feed_url );
        xml_parse_into_struct( $xmlParser, $xmlData, $dataStruct, $dataIndex );
        xml_parser_free( $xmlParser );

        // Set up the data nicely in an array first //
        $artistDataArray = array ();

        // Setup the Artist Information //
        $artistName = $dataStruct[$dataIndex["title"][0]]['value'];
        $artistUrl = $dataStruct[$dataIndex["link"][0]]['value'];

        // Set Description //
        $descriptionContent = $dataStruct[$dataIndex["description"][0]]['value'];

        // Loop through all the occurences of item tag index values //
        foreach ( $dataIndex["item"] as $itemOpenIndex=>$itemIndex )
        {
            // Check if this occurence is a start tag or a close tag //
            if ( $dataStruct[$itemIndex]['type'] == "open" )
            {
                // Get the index where this item closes, this is generally the next occurence of the tag //
                $itemCloseIndex = $dataIndex["item"][$itemOpenIndex+1];

                // Setup temp item array //
                $tempItemArray = array ();

                // Loop through all the index between the item open and close and add that to the data array //
                for ( $i=$itemOpenIndex+1; $i<$itemCloseIndex; $i++ )
                {
                    // Add the data to the item array //
                    $tempItemArray[$dataStruct[$i]['tag']] = $dataStruct[$i]['value'];
                }

                // Add the item to the data array //
                $artistDataArray[] = $tempItemArray;
            }
        }

        // Now set the new cached data //
        delete_option( "widget_ubw_datacache" );

        // Create new object to add to database //
        $newCache = new stdClass;
        $newCache->time = time();
        $newCache->data = $artistDataArray;
        $newCache->artist_name = $artistName;
        $newCache->artist_url = $artistUrl;
        $newCache->description = $dataStruct[$dataIndex["description"][0]]['value'];

        // Update the option in the database //
        update_option( "widget_ubw_datacache", $newCache );
    }

    // Create the return string with all the data //
    $returnString = '';
    

    // Start the Listing //
    $returnString .= '<ul class="ubwWidgetList">';

    $returnString .= '<li class="ubwWidgetDescription">feed: <a href="' . $artistUrl . '">' . $artistName . '</a></li>';

    // Loop through all the items we collected and get the required amount //
    foreach ( $artistDataArray as $index=>$item )
    {
        // Add only as many items as needed and setup in the options //
        if ( ( $index+1 ) <= $options->display_limit )
        {
            // Do the mod for item title //
            $tempTitle = $item["title"];

            // Explode it by - //
            $tempTitleParts = explode( "-", $tempTitle );

            // If we have more than two values, then it means the title had a - in it, so handle that properly //
            if ( isset( $tempTitleParts[2] ) )
            {
                // Get the last element out of the way //
                array_pop( $tempTitleParts );

                // Add the title now //
                $item["title"] = trim( implode( "-", $tempTitleParts ) );
            }
            // Just two parts we are lucky //
            else
            {
                // Check to make we at least have two parts //
                if ( !isset( $tempTitleParts[1] ) )
                    $item["title"] = trim( $tempTitleParts[0] );
                // No two parts either, the title is fine as it is //
                else
                    $item["title"] = $tempTitleParts[0];
            }

            // Get the band id and song id from the link //
            $urlParts = parse_url( $item['link'] );
            $queryParts = explode( "&", $urlParts['query'] );
            // Add the item to the list //
            $returnString .= '<li>
                                ' . ( $options->display_image == 1 ? '<img src="http://www.unsignedbandweb.com/music/image.php?mode=song_image&' . $queryParts[1] . '&' . $queryParts[2] . '&width=40&height=40" class="ubwWidgetImage" alt="' . $artistName . '" />' : '' ) . '
                                <a href="' . $item['link'] . '" title="Play Song - ' . $item['title'] . '">"' . substr(trim($item['title']), 0, 15) . '"</a>
                                <br />
                                <a href="' . $artistUrl . '" title="Go to the website for the band - ' . $artistName . '"><i>' . substr($artistName, 0, 15) . '</i></a>
                                <br />
                                <a href="' . $item['link'] . '" title="Play Song - ' . $item['title'] . '">[ Play ]</a><a onclick="windowOpen(this.href, \'name\', \'430\', \'500\', \'yes\'); return false;" href="' . $item['comments'] . '" title="Comments for Song - ' . $item['title'] . '">[ Talk ]</a>
                            </li>';
        }
        // Reached the limit for display, break out //
        else
            break;
    }

    // Add link back //
    $returnString .= '<li class="ubwWidgetLinkBack">Powered By <a href="http://www.unsignedbandweb.com/">Free Music</a></li>';

    // End the list //
    $returnString .= '</ul>';

    // Return the data now to be displayed in the widget //
    return $returnString;
}


/*
*	Widget Javascript
*/

function widget_wpubwInitJavascriptAndStyle ()
{
    $javascript = "<script type='text/javascript'>
                    /* <![CDATA[ */
                    function windowOpen ( mypage, myname, w, h, scroll )
                    {
                        LeftPosition = (screen.width) ? (screen.width-w)/2 : 0;
                        TopPosition = (screen.height) ? (screen.height-h)/2 : 0;
                        settings = 'height='+h+',width='+w+',top='+TopPosition+',left='+LeftPosition+',scrollbars='+scroll+',resizable';
                        win = window.open( mypage, myname, settings );
                        if ( win.opener == null ) {
                            win.opener = self;
                        }
                    }
                    /* ]]> */
                    </script>";

    $javascript .= "<style type='text/css'>
                        ul.ubwWidgetList {
                            margin: 0px;
                            padding: 0px;
                            font-size: 11px;
                            list-style-type: none;
                        }
                        ul.ubwWidgetList li {
                            clear: both;
                            display: block;
                            font-size: 11px;
                            padding-bottom: 15px;
                            line-height: 15px;
                        }
                        ul.ubwWidgetList li a {
                            text-decoration: none;
                            font-weight: normal;
                            font-size: 11px;
                        }
                        ul.ubwWidgetList li.ubwWidgetLinkBack {
                            text-align: left;
                        }
                        ul.ubwWidgetList li.ubwWidgetDescription {
                            padding-bottom: 5px !important;
                            font-size: 11px;
                        }
                        img.ubwWidgetImage {
                            float: left;
                            margin-right: 5px;
                            margin-bottom: 12px; 
                            border: 3px solid #CCC;
                        }
                        .noimagehr {
                           width: 80%;
                           line-height: 4px;
                           color: #DDD;
                        }
                    </style>";

    echo $javascript;
}

add_action( "wp_head", "widget_wpubwInitJavascriptAndStyle" );

?>